// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *			   XVT Service classes
 *
 * directories, lists of strings, etc.
 *
 * $Id: xvt-serv.h,v 1.3 1994/06/28 21:34:20 oleg Exp oleg $
 *
 *************************************************************************
 */

#ifndef __GNUC__
#pragma once
#else
#pragma interface
#endif

#ifndef _xvt_serv_h
#define _xvt_serv_h 1

#include <xvt.h>
  
			// When this object is created, it remembers
			// the current directory. Then it's always
			// possible to return to that directory when
			// the object is destroyed, or on explicit request
class HomeDirectory
{
  DIRECTORY dir;
public:
  HomeDirectory(void)		{ assert( get_dir(&dir) ); }
  ~HomeDirectory(void)		{ come_home(); }
  void come_home(void) const	{ assert( chg_dir((DIRECTORY *)&dir) ); }
};


class XVTBasicWindow;
			// List of strings: each element of the list contains
			// a string (residing in its own memory) and a key
			// (arbitrary long int)
class ListofStrings
{
  friend XVTBasicWindow;

  SLIST list;
  SLIST_ELT curr_elem;

protected:	
  void set(SLIST a_list)
	{ assert( slist_valid(a_list) ); slist_dispose(list); list=a_list; }

public:
  ListofStrings(void)	{ assert( (list = slist_new()) ); curr_elem=0; }
  ~ListofStrings(void)	{ assert( slist_valid(list) ); slist_dispose(list); }
  void append(const char * str, const long key = 0)
  			{ assert( slist_valid(list) );
			  slist_add(list,0,(char *)str,key); }
  int no_elems(void) const
		{ assert( slist_valid(list) ); return slist_count(list); }

  				// Iterators
  void first(void)
	{ assert( slist_valid(list) ); curr_elem = slist_first(list); }
  BOOLEAN next(void)
	{ assert( slist_valid(list) );
	  curr_elem = slist_next(list,curr_elem); return curr_elem != 0; }

				// Get info from the curr element
  operator BOOLEAN (void) const		{ return curr_elem != 0; }
  const char * curr_str(void) const
	{ assert(curr_elem != 0); long temp;
	  return slist_get(list,curr_elem,&temp); }
  long& curr_key(void) const
	{ assert(curr_elem != 0); return *slist_data(curr_elem); }
			// Replace the data in the current element 
  void curr_replace(const char * str, const long key = 0);
  
			// Remove the current element 
			// (no current elem after that!)
  void remove(void)
	{ assert( curr_elem != 0 );
	  assert( slist_rem(list,curr_elem) ); curr_elem = 0; }
  
			// Get info from the element with a given index
  const char * str_by_index(const short index) const
	{ assert(slist_valid(list)); long temp;
	  return slist_elt(list,index,&temp); }
  long key_by_index(const short index) const
	{ assert(slist_valid(list)); long temp;
	  assert(slist_elt(list,index,&temp)); return temp; }
			// Set a current elmt by its index
  void set_curr_by_index(const short index);
  
  void dump(void)	{ assert( slist_valid(list) ); slist_dbg(list); }
};


class ListofFileNames : public ListofStrings
{
public:
  ListofFileNames(void) {}
  ListofFileNames(char * pattern, BOOLEAN include_dirs = FALSE)
		{ list_files(pattern,include_dirs); }
  ~ListofFileNames(void) {}
  void list_subdirs(char * pattern="")
		{ set(::list_files(DIR_TYPE,pattern,TRUE)); }
  void list_files(char * pattern, BOOLEAN include_dirs = FALSE)
		{ set(::list_files("",pattern,include_dirs)); }
};

#endif
