// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *		     Color Lookup Manipulation Classes
 *
 * Implementation of service color manipulation (and color description
 * parsing) functions and saving/restoring/changing ranges of gray CLUT.
 *
 * $Id: clut_manip.cc,v 1.1 1994/10/03 13:59:21 oleg Exp oleg $
 *
 ***********************************************************************
 */

#ifdef __GNUC__
#pragma implementation
#endif

#include "clut_manip.h"
#include <std.h>

/*
 *------------------------------------------------------------------------
 *			Color Manipulation Functions
 */
 
				// Parse the color specification string
				// At present, it's supposed to be
				// #rgb or #rrggbb,
				// where r, g, and b are hex digits
XVTColor::XVTColor(const char * rgb_str)
{
  char * p = strchr(rgb_str,'#');
  if( p == 0 )
    xvt_fatal("Color spec string '%s' is invalid "
	      "because it doesn't contain #",rgb_str);

  char * str_end;
  unsigned long rgb_spec = strtol(++p,&str_end,16);
  switch(int str_len = str_end - p)
  {
    case 3:				// it was #rgb spec
         red = (rgb_spec >> 8) & 0x0f;
	 green = (rgb_spec >> 4) & 0x0f;
	 blue  = rgb_spec & 0x0f;
         break;

    case 6:				// it was #rrggbb spec
         red = (rgb_spec >> 16) & 0xff;
	 green = (rgb_spec >> 8) & 0xff;
	 blue  = rgb_spec & 0xff;
         break;

    default:
	 xvt_fatal("Color spec string '%s' is invalid "
		   "because it contains %d hex digits; "
		   "only 3 or 6 is expected",rgb_str,str_len);
  }
}

/*
 *------------------------------------------------------------------------
 *			Manipulating Ranges of gray CLUT
 */
 
			// Replace grayshades for pixels from 'lo' up to 'hi'
			// with a given color, thus pseudo-colorizing
			// (highlighting) these pixels on whatever images are
			// displayed out there
			// The grayshades are saved and can be restored later

PseudocolorCLUTRange::PseudocolorCLUTRange
	(const GRAY lo, const GRAY hi, const XVTColor pseudocolor)
{
  assert( lo <= hi );
  
  cmap_range = 1;		// Count how many clut entries are to be
				// modified
  GRAYSHADE shade = grayshade_of(lo);
  for(register GRAY gray=lo+1; gray <=hi; gray++)
  { 
    const GRAYSHADE new_shade = grayshade_of(gray);
    if( new_shade != shade )
      shade = new_shade, cmap_range++;
  }
  cmap_saved_range = new GrayCLUTEntry[cmap_range];

				// Saving graymap shades (contents of CLUT
				// entries)
  GrayCLUTEntry * entries = cmap_saved_range;
  shade = grayshade_of(lo)-1;	// to make it different from grayshade_of(lo)
  for(gray=lo; gray <=hi; gray++)
  {
    const GRAYSHADE new_shade = grayshade_of(gray);
    if( new_shade == shade )
      continue;
    shade = new_shade;
    GrayCLUTEntry& entry = *entries++;
    entry.index = shade;
    entry.gray_contents = get_entry(entry.index);
    set_entry(entry.index,pseudocolor);
  }
  assert( entries == cmap_saved_range + cmap_range );
}


		// This would unpseudocolorize affected
		// pixels
PseudocolorCLUTRange::~PseudocolorCLUTRange(void)
{
  assert( cmap_range > 0 && cmap_saved_range != 0 );

  for(register GrayCLUTEntry * entries = cmap_saved_range;
      entries < cmap_saved_range + cmap_range; entries++)
  {
    GrayCLUTEntry& entry = *entries;
    set_entry(entry.index,entry.gray_contents);
  }
    
  delete [] cmap_saved_range; cmap_saved_range = 0;
}


