// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *			  	CLUT strip
 *
 * This simple tool shows the gray-shades available for displaying
 * gray-scale images, that is the "gray" portion of the current
 * CLUT (color look-up table). It also lets the user to change some
 * portions of CLUT, that is, replace some grays with pseudo-color.
 *
 * This strip should be used with modeless dialog, because it merely
 * shows the window but doesn't wait until the window gets closed.
 * Don't forget to call serve_to_death() somewhere.
 *
 * Also, don't forget to include a "string" resource labelled IDS_CLUTSTRPC
 * in the resource "fork" of the application. That resource string
 * describes the color (in #rgb or #rrggbb formats) to use in pseudo-
 * coloring. Usually, #including the file 'clut_strip.url' into your
 * application's URL will do the trick.
 *
 * $Id: clut_strip.cc,v 1.1 1994/10/03 14:06:33 oleg Exp oleg $
 *
 ***********************************************************************
 */

#include "clut_manip.h"
#include "clutstr_intf.h"


				// Create the gray strip and a window to
				// view it
CLUTStrip::CLUTStrip(const char * title)
	: XVTImageWindow(IDS_CLUTSTRIP,1,256,title,TASK_WIN),
	  pseudo_coloring_width(10),
	  pseudocolorized_range(0),
	  pseudo_color(0,0,0)
{
  for(register int j=0; j<q_ncols(); j++)
    (*this)(0,j) = j;				// Make the grey strip
  refresh();
  redraw();
  show();

  char rgb_str[30];
  if( get_res_str(IDS_CLUTSTRPC,rgb_str,sizeof(rgb_str)) == 0 )
    xvt_fatal("Failed to get pseudocolor spec string from the resource");

  pseudo_color = XVTColor(rgb_str);
}

				// Uncolorize the grayscale if it was
				// colorized
CLUTStrip::~CLUTStrip(void)
{
  if( pseudocolorized_range != 0 )
    delete pseudocolorized_range;
  pseudocolorized_range = 0;
}

				// It handles in particular only the E_COMMAND
				// event, that is, a menu selection
				// and mouse pressing events
				// The rest of events are handled by a generic
				// window event handler
void CLUTStrip::handler(EVENT event)
{
  if( event.type == E_COMMAND )
    switch(event.v.cmd.tag)
    {
      case M_FILE_CLOSE:
           delete this;			// Rather risky!
	   return;

      case M_FILE_QUIT:
           xvt_terminate();
	   break;

#if 0
      case IM_INVERT:
	   IMAGE::invert();
	   refresh();
	   redraw();
	   break;
#endif

      default:
	   break;
  }
  else if( event.type == E_MOUSE_DOWN )
    handle_mouse_click(event.v.mouse.where);

  XVTImageWindow::handler(event);
}

				// Handle the mouse click within the strip
				// The point 'where' (which is in relative
				// coordinates) is guaranteed to be within
				// the window (otherwise, the window
				// wouldn't've gotten the mouse event
				// in the first place). The program
				// colors the band of some width around
				// the selected gray shade by replacing the
				// band with the pseudo-color
void CLUTStrip::handle_mouse_click(const PNT where)
{
  int gray_shade = ((long)where.h * q_ncols()) /
    		   ((XVTRect)(XVTBasicWindow&)(*this)).q_width();

  if( gray_shade < 0 )			// Do some clipping, though it's
    gray_shade = 0;			// probably redundant since 'where'
  if( gray_shade >= q_ncols() )		// ought to be already within the
    gray_shade = q_ncols()-1;		// window boundaries

  int lo_bound = max(0,gray_shade - pseudo_coloring_width/2);
  int hi_bound = min(q_ncols()-1,lo_bound + pseudo_coloring_width);

  if( pseudocolorized_range != 0 )
    delete pseudocolorized_range;

  pseudocolorized_range = new PseudocolorCLUTRange(lo_bound,hi_bound,
						   pseudo_color);
//  redraw();
  draw_immediately();
}

